from .geom import *
from .polygon import Polygon
from collections import deque

def segments_from_intersection(polygon1, polygon2):
    segment_list = []

    for v1 in polygon1:
        intersections = []
        a1, a2 = v1.point, v1.next().point
        for v2 in polygon2:
            b1, b2 = v2.point, v2.next().point

            pos = segment_position_segment(v1.point, v1.next(), v2.point, v2.next())

            if pos == SEGMENT_INTERSECT:
                p = intersection_point(a1, a2, b1, b2)

                if not point_equals(a1, p) or not point_equals(a2, p):
                    intersections.append(p)

            elif pos == SEGMENT_COLLINEAR:
                if point_in_segment(b1, a1, a2):
                    intersections.append(b1)

                if point_in_segment(b2, a1, a2):
                    intersections.append(b2)

        intersections = sort_points_distance(a1, intersections)
        while len(intersections)>0 and point_equals(intersections[-1], a2):
            intersections = intersections[:-1]

        while len(intersections)>0 and point_equals(intersections[0], a1):
            intersections = intersections[1:]

        if len(intersections)>0:
            segment_list.append((a1, intersections[0]))
            for p in range(1, len(intersections)):
                if not point_equals(intersections[p-1], intersections[p]):
                    segment_list.append((intersections[p-1], intersections[p]))

            segment_list.append((intersections[-1], a2))
        else:
            segment_list.append((a1, a2))

    return segment_list



def boolean_operation(polygon1, polygon2, function):
    segments1 = segments_from_intersection(polygon1, polygon2)
    segments2 = segments_from_intersection(polygon2, polygon1)

    result_segments = []

    for seg in segments1:
        s = function(1, polygon2, seg)
        if s:
            result_segments.append(s)

    for seg in segments2:
        s = function(2, polygon1, seg)
        if s:
            result_segments.append(s)

    return result_segments


def polygon_intersection(polygon1, polygon2):
    def intersection_function(pol, polygon, segment):
        if polygon.point_in_polygon(middle(*segment)):
            return segment

    return boolean_operation(polygon1, polygon2, intersection_function)

def polygon_union(polygon1, polygon2):
    def union_function(pol, polygon, segment):
        if polygon.point_in_polygon(middle(*segment)) != 2:
            #If border or outside
            return segment

    return boolean_operation(polygon1, polygon2, union_function)

def polygon_difference(polygon1, polygon2):
    def difference_function(pol, polygon, segment):
        if pol == 1 and polygon.point_in_polygon(middle(*segment)) != 2:
            return segment
        elif pol == 2 and polygon.point_in_polygon(middle(*segment)):
            return (segment[1], segment[0])

    return boolean_operation(polygon1, polygon2, difference_function)

def polygon_reconstruction(segments):
    """
    Returns a list of polygons. If polygon is clockwise it is a hole.
    """
    ss = deque()
    result = []

    for p1, p2 in segments:
        #Is it a point??
        if not point_equals(p1, p2):
            add = True
            for t1, t2 in segments:
                if point_equals(p1, t2) and point_equals(p2, t1):
                    add = False
                    break

            if add: ss.append((p1, p2))

    while len(ss)>0:
        s1, s2 = ss.popleft()
        f1 = s1

        current_polygon = [s1]

        stop = False

        while not stop:
            #Loop to build current polygon
            candidate = None
            sin = 10000

            for n in range(len(ss)):
                #Loop to find next vertex
                t1, t2 = ss.popleft()

                if point_equals(s2, t1):
                    a = sin_angle((s1[0]-s2[0], s1[1]-s2[1]), (t2[0]-t1[0], t2[1]-t1[1]))
                    if a < sin:
                        if candidate is not None: ss.append(candidate)
                        candidate = (t1, t2)
                        sin = a
                    else:
                        ss.append((t1, t2))
                else:
                    ss.append((t1, t2))


            if  candidate is None and point_equals(s2, f1):
                stop = True
                result.append(Polygon(current_polygon))
            elif candidate is None:
                result.append(Polygon(current_polygon))
                stop = True
            else:
                current_polygon.append(candidate[0])
                s1, s2 = candidate


            if candidate in ss:
                ss.remove(candidate)


    return result
